from picsl_c3d import Convert3D
import math
import argparse
import os
import json
import io
import numpy as np
import SimpleITK as sitk

class Parameters:
    def __init__(self):
        # Things related to input and output files
        self.fn_input_mri = ""
        self.fn_input_seg = ""
        self.fn_input_dots = ""
        self.fn_input_avoidance = ""
        self.fn_output_root = ""
        self.id_string = ""

        # Side of the hemisphere
        self.side = 'N'

        # Parameters for mold creation
        self.slit_spacing_mm = 10.0
        self.slit_width_mm = 1.6
        self.mold_resolution_mm = 0.4
        self.mold_wall_thickness_mm = 3
        self.mold_floor_thickness_mm = 3

        # The amount of dilation and erosion applied to the mask as a
        # preprocessing step
        self.preproc_dilation = 5
        self.preproc_erosion = 3
        
        # Depth at which the guide is generated
        self.dot_guide_depth = None

        # Flags
        self.flag_print_hemisphere_mold = True

        # Specific slab to analyze
        self.selected_slab = -1
        
    def to_dict(self):
        
        def rebase(k, v):
            if k.startswith('fn_') and k != 'fn_output_root' and v is not None and os.path.exists(v):
                print(k, v)
                return os.path.relpath(v, self.fn_output_root)
            else:
                return v
        
        return { k : rebase(k,v) for k, v in self.__dict__.items()}
    
    def from_dict(self, d):        
        for k, v in self.__dict__.items():
            if k in d:
                v = d[k]
                if k.startswith('fn_') and k != 'fn_output_root':
                    v = os.path.abspath(os.path.join(self.fn_output_root, v))
                self.__dict__[k] = v
        
    def _generate_filename(self, subdir, suffix, slab=None):
        """Helper function to generate filenames and create directories."""
        # Generate the base filename
        if slab is not None:
            filename = f"{self.id_string}_slab{slab:02d}_{suffix}"
        else:
            filename = f"{self.id_string}_{suffix}"

        # Full path for the output directory
        if subdir:
            filepath = os.path.join(self.fn_output_root, subdir, filename)
        else:
            filepath = os.path.join(self.fn_output_root, filename)

        # Create the directory if it doesn't exist
        os.makedirs(os.path.dirname(filepath), exist_ok=True)
        return filepath

    def fn_hemi_mold_image(self):
        return self._generate_filename(None, "hemi_mold.nii.gz")

    def fn_hemi_volume_image(self):
        return self._generate_filename(None, "hemi_volume.nii.gz")

    def fn_dot_guide_image(self):
        return self._generate_filename(None, "dots_guide.nii.gz")

    def fn_hemi_c3d_log(self):
        return self._generate_filename("logs", "hemi_c3d_log.txt")

    def fn_slab_mold_image(self, slab):
        return self._generate_filename(None, "mold.nii.gz", slab)

    def fn_slab_volume_image(self, slab):
        return self._generate_filename(None, "volume.nii.gz", slab)

    def fn_config_json(self):
        return os.path.join(self.fn_output_root, 'config.json')

    def fn_slab_c3d_log(self, slab):
        return self._generate_filename("logs", "c3d_log.txt", slab)

    def fn_slab_optimization_log(self, slab):
        return self._generate_filename("logs", "opt_log.txt", slab)

    def fn_slab_cutplane_json(self, slab):
        return self._generate_filename(None, "cutplanes.json", slab)

    def fn_slab_cut_png(self, slab):
        return self._generate_filename("tex", "cuts.png", slab)

    def fn_slab_cut_texlet(self, slab):
        return self._generate_filename("tex", "cuts.tex", slab)

    def fn_slab_with_dots_volume_image(self, slab):
        return self._generate_filename(None, "mask_with_dots.nii.gz", slab)

    def fn_slab_tex_png(self, slab, flip):
        return self._generate_filename("tex", f"template_{flip}.png", slab)

    def fn_global_texfile(self):
        return self._generate_filename("tex", "print_template_no_cuts.tex")


def make_mold(id:str, side:str, 
              fn_input_mri:str, fn_input_seg:str, fn_output_root:str, 
              dot_guide_depth=None):
    
    # Set up the parameters data structure
    param = Parameters()
    param.id_string, param.side = id, side
    param.fn_input_mri = fn_input_mri
    param.fn_input_seg = fn_input_seg
    param.fn_output_root = fn_output_root
    param.dot_guide_depth = dot_guide_depth

    with open(param.fn_hemi_c3d_log(), 'wt') as fn_log:
        
        # Initialize API and redirect output
        api = Convert3D(out=fn_log, err=fn_log)

        # Read the mask image and dots, avoidance images
        api.execute(f"-verbose {param.fn_input_seg} -popas mask")

        if param.fn_input_dots:
            api.execute(f"{param.fn_input_dots} -swapdim LPI -popas dots")
        else:
            api.execute("-push mask -scale 0 -swapdim LPI -popas dots")

        if param.fn_input_avoidance:
            api.execute(f"{param.fn_input_avoidance} -swapdim LPI -popas avoid")
        else:
            api.execute("-push mask -scale 0 -swapdim LPI -popas avoid")

        # Trim and resample hemisphere
        trim_radius = max(param.mold_wall_thickness_mm, max(param.mold_floor_thickness_mm, 5.))
        print("Extracting hemisphere main connected component and resampling")

        r_dil, r_ero = param.preproc_dilation, param.preproc_erosion
        api.execute(f"-push mask -swapdim LPI -thresh 1 1 1 0 -as comp_raw "
                    f"-dilate 1 {r_dil}x{r_dil}x{r_dil} -dilate 0 {r_ero}x{r_ero}x{r_ero} -as comp "
                    f"-trim {trim_radius}mm -resample-mm {param.mold_resolution_mm}mm -as H")

        # Extract image details
        i_comp_resampled = api.get_image('H')
        sz = i_comp_resampled.GetSize()
        sp = i_comp_resampled.GetSpacing()
        off_y = (sz[1] * sp[1] / 2 - 5)

        # Compute slab positions
        idx_center = [ sz[i] * 0.5 for i in range(3)]
        p_center = i_comp_resampled.TransformContinuousIndexToPhysicalPoint(idx_center)
        y_center = -p_center[1]

        # Number of slabs
        n_slabs = int((off_y / param.slit_spacing_mm) * 2)
        print(f"Slabs {n_slabs}, center slab coordinates: {y_center}")

        slabs = [{'y0': y_center + (n_slabs / 2. - i) * param.slit_spacing_mm - param.slit_spacing_mm,
                'y1': y_center + (n_slabs / 2. - i) * param.slit_spacing_mm}
                for i in range(n_slabs)]

        # Print the main mold if requested
        if param.flag_print_hemisphere_mold:
            print("Generating hemisphere mold")
            api.execute("-clear -push H -cmp -popas z -popas y -popas x "
                        "-push H -scale 0 -shift 4")

            for i in range(n_slabs + 1):
                y_cut = slabs[i]['y0'] if i < n_slabs else slabs[i - 1]['y1']
                api.execute(f"-push y -shift {-y_cut} -abs -stretch 0 {param.slit_width_mm} -4 4 -clip -4 4 -min")

            api.execute(f"-as res1 -o {param.fn_hemi_volume_image()}")

            ext_dir_1 = "RPS" if param.side == 'R' else "LPS"
            ext_dir_2 = "LPS" if param.side == 'R' else "RPS"
            floor_offset_vox = (trim_radius - param.mold_floor_thickness_mm) / param.mold_resolution_mm
            api.execute(f"-clear -push H -swapdim {ext_dir_1} -cmv -pick 0 "
                        f"-stretch {floor_offset_vox - 1.0} {floor_offset_vox + 1.0} 4 -4 -clip -4 4 -swapdim LPI -as base")

            api.execute("-clear -push H -push base -push res1 -max "
                        "-copy-transform -pad 5x5x5 5x5x5 -4 -as mold")

            p3 = math.ceil(param.mold_wall_thickness_mm / param.mold_resolution_mm)
            api.execute(f"-clear -push comp -thresh -inf 0.5 4 -4 "
                        f"-swapdim {ext_dir_1} -extrude-seg -swapdim LPI -dup "
                        f"-swapdim {ext_dir_2} -extrude-seg "
                        f"-thresh -inf 0 0 1 -dilate 0 0x{p3}x{p3} -stretch 0 1 4 -4 -swapdim LPI -min "
                        f"-insert mold 1 -background 4 -reslice-identity "
                        f"-push mold -min -as carved -thresh 0 inf 1 0")

            # Save final output
            api.execute(f"-clear -push comp -o {param.fn_hemi_volume_image()}")
            api.execute(f"-push carved -o {param.fn_hemi_mold_image()}")
            
        # Generate the dots guide
        if param.dot_guide_depth:
            
            # Take the original image, the guides will be defined in this space. But we need to 
            # swapdim the image so we know what direction AP corresponds to
            # api.execute('-push mask -swapdim LPS -scale 0')
            api.execute(f'{param.fn_input_seg} -as src -swapdim LPS -o goo.nii.gz -scale 0')
            i_guide = api.peek(-1)
            
            # For each slab, tag a plane
            for i, s in enumerate(slabs):
                y_phy = s['y0'] - param.dot_guide_depth
                y_img = int(0.5 + i_guide.TransformPhysicalPointToContinuousIndex([0, -y_phy, 0])[1])
                print(f"Slab {i} : [{s['y0']:5.2f} - {s['y1']:5.2f}] marking slice y={y_img}")
                i_guide[:,y_img,:] = 128
                
            # Save the guide image
            api.push(i_guide)
            api.execute(f'-insert mask 1 -int 0 -reslice-identity -push mask -dilate 1 3x3x3 -thresh 0 0 1 0 -times '
                        f'-type uchar -o {param.fn_dot_guide_image()}')
            
            # Generate a file with the parameters and slab information
            state_dict = {
                'slabs': slabs,
                'param': param.to_dict()
            }
            with open(param.fn_config_json(), 'w') as fd:
                json.dump(state_dict, fd, indent=4, sort_keys=True)
            

def process_slab(param:Parameters, slab_id:int, slab: dict, 
                 i_hemi_raw, i_hemi_mask, i_dots, dot_coord):
    
    with open(param.fn_slab_c3d_log(slab_id), 'wt') as fn_log:
    
        # Create our own API
        api = Convert3D(out = fn_log, err = fn_log)
    
        # Extract the slab image region
        api.add_image("H", i_hemi_mask)
        api.add_image("dots", i_dots)
        api.execute(
            f"-verbose -clear -push H -cmp -pick 1 -thresh {slab['y0']} {slab['y1']} 255 0 "
            f"-push H -times -thresh 0 127 0 255 -info -trim 0vox -as slab ")
        
        # Check if the image is empty
        i_slab = api.get_image('slab')
        if i_slab.GetNumberOfPixels() == 0:
            print("  Empty slab encountered")
            return
        
        # Process each dot stat
        for dot_value, dot_center in dot_coord.items():

            # Check if the dot exists on this slide
            if dot_center[1] >= slab['y0'] and dot_center[1] <= slab['y1']:

                print(f"  Slab {slab_id} contains dot {dot_value} with physical coordinates {dot_center}")

                # Mark the dot with a 3x3x3 cube in the target volume
                api.execute(f'-clear -push slab -dup -push dots -thresh {dot_value} {dot_value} 1 0 '
                            f'-dilate 1 1x1x1 -dup -lstat -pop -centroid -int 0 -reslice-identity '
                            f'-centroid-mark {dot_value} -dilate {dot_value} 2x2x2 -composite -as slab')
            
        # Save the image
        output_filename = param.fn_slab_with_dots_volume_image(slab_id)
        api.execute(f"-push slab -type uchar -o {output_filename}")        
        
        
def read_config(fn_output_root:str):
    
    # Read the configuration and slab indices
    param = Parameters()
    param.fn_output_root = fn_output_root

    slabs = []
    with open(param.fn_config_json(), 'r') as fd:
        d = json.load(fd)
        try:
            param.from_dict(d['param'])
            slabs = d['slabs']
        except:
            print(f"Failed to read configuration from {param.fn_config_json()}")
    
    return param, slabs    


def make_slabs(fn_output_root:str, fn_dots:str, slab_index=None):
    
    # Read the configuration and extra parameter
    param, slabs = read_config(fn_output_root)
    param.fn_input_dots = fn_dots
        
    # Set up Convert3D
    sout = io.StringIO()
    api = Convert3D(out = sout)
    
    # Generate the necessary inputs to the slab extractor
    api.execute(f'{param.fn_input_seg} -swapdim LPI -thresh 1 1 1 0')
    i_comp_raw = api.peek(-1)
    trim_radius = max(param.mold_wall_thickness_mm, max(param.mold_floor_thickness_mm, 5.))
    api.execute(f'{param.fn_hemi_volume_image()} '
                f'-trim {trim_radius}mm -resample-mm {param.mold_resolution_mm}mm ')
    i_comp_resampled = api.peek(-1)
    api.execute(f'{param.fn_input_dots} -as D')
    i_dots = api.peek(-1)
    
    # We need to find the centroid of each dot in physical units
    a_dots = sitk.GetArrayFromImage(i_dots)
    dot_val = np.setdiff1d(np.unique(a_dots), [0., 128.])
    dot_coord = {}
    for i in dot_val:
        dot_index = np.stack(np.nonzero(a_dots == i), 1)
        dot_center_index = np.mean(dot_index, 0)
        dot_coord_lps = i_dots.TransformContinuousIndexToPhysicalPoint(np.flip(dot_center_index))
        dot_coord[i] = [ -x if i < 2 else x for (i, x) in enumerate(dot_coord_lps) ]
        print(f'Dot {i} has center {dot_coord[i]} in Nifti RAS physical coordinates')

    for i, slab in enumerate(slabs):
        if slab_index is None or i == slab_index:
            print(f'Generating mold for Slab {i:02d} of {len(slabs)} with range [{slab["y0"]:5.2f}, {slab["y1"]:5.2f}]')
            process_slab(param, i, slabs[i], i_comp_raw, i_comp_resampled, i_dots, dot_coord)
        


class BrainMoldLauncher:
    
    def __init__(self, parser):
        
        # Required arguments
        parser.add_argument("--subject", "-s", required=True, help="Subject ID, used in naming files")
        parser.add_argument("--hemisphere", "-H", choices=["L", "R"], required=True, help="Which hemisphere (left or right)")
        parser.add_argument("--input", "-i", nargs=2, required=True, metavar=('mri', 'mask'), help="Input MRI and brain mask")
        parser.add_argument("--work", "-w", required=True, help="Work directory for brainmold (will be created)")
        
        # Optional arguments
        parser.add_argument("--guide", "-g", default=None, type=float, metavar='depth',
                            help="Generate guide images for dot placement at given depth from the anterior, in mm")

        # Optional arguments
        # parser.add_argument("--dots", "-d", help="Dots segmentation image")
        # parser.add_argument("--avoidance", "-a", help="Avoidance segmentation image")
        # parser.add_argument("--no-hemi-mold", action="store_true", help="Skip hemisphere mold printing")
        # parser.add_argument("--slab", type=int, default=-1, help="Only generate for one slab (negative for none)")
        
        parser.set_defaults(func = lambda args : self.run(args))
        
    def run(self, args):
        
        # Set the parameters from arguments
        make_mold(id = args.subject, side=args.hemisphere,
                  fn_input_mri = args.input[0], 
                  fn_input_seg = args.input[1],
                  fn_output_root = args.work,
                  dot_guide_depth = args.guide)
        
        
class SlabGeneratorLauncher:
    
    def __init__(self, parser):
        
        # Required arguments
        parser.add_argument("--work", "-w", required=True, help="Work directory for brainmold (will be created)")
        parser.add_argument("--dots", "-d", required=True, help="Dots segmentation image")        
        parser.set_defaults(func = lambda args : self.run(args))
        
    def run(self, args):
        
        # Set the parameters from arguments
        make_slabs(fn_output_root = args.work, fn_dots = args.dots)




import argparse
from picsl_brainmold.brainmold import BrainMoldLauncher, SlabGeneratorLauncher
from picsl_brainmold.slab_postproc import PaperTemplateLauncher

# Create a parser
parse = argparse.ArgumentParser(
    prog="picsl_brainmold", description="PICSL BrainMold: Generator for 3D Printed Brain Molds")

# Add subparsers for the main commands
sub = parse.add_subparsers(dest='command', help='sub-command help', required=True)

# Add the CRASHS subparser commands
c_bm = BrainMoldLauncher(
    sub.add_parser('mold', help='Generate a cutting mold for a hemisphere'))

c_sg = SlabGeneratorLauncher(
    sub.add_parser('slabs', help='Generate masks for individual slabs'))

c_paper = PaperTemplateLauncher(
    sub.add_parser('paper', help='Generate a paper template for block cutting'))

# Parse the arguments
args = parse.parse_args()
args.func(args)



import SimpleITK as sitk
import matplotlib.pyplot as plt
import matplotlib.patheffects as pe
import matplotlib.ticker
# import nibabel as nib
import numpy as np
import argparse
import glob
import os
import pathlib
import subprocess
import textwrap
from picsl_c3d import Convert3D

from picsl_brainmold.brainmold import Parameters, read_config

def make_paper_template(fn_output_root:str):
    
    # Read the configuration and extra parameter
    param, slabs = read_config(fn_output_root)

    # Find all the matching files in the directory
    slab_indices = []
    for i,_ in enumerate(slabs):
        if os.path.exists(param.fn_slab_with_dots_volume_image(i)):
            slab_indices.append(i)

    print(f"Found {len(slab_indices)} input files in {param.fn_output_root}")

    # Latex strings
    s_preamble = """
        \\documentclass{article}
        \\usepackage[table]{xcolor}
        \\usepackage{graphicx}
        \\usepackage[letterpaper, margin=0.25in]{geometry}
        \\renewcommand{\\arraystretch}{1.5}
        \\setlength\\tabcolsep{0.25in}
        \\begin{document}
        """

    s_slab = """
        \\begin{figure}
        \\centering
        {\\centering \\Large \\textbf{Specimen %s slab %02d}}
        \\newline \\newline
        \\includegraphics[width=%fin]{%s}
        \\hfill \\includegraphics[width=%fin]{%s}
        \\newline \\newline
        %s
        \\end{figure}
        """

    s_closing = """
        \\end{document}
        """

    # Label names
    dot_names = {
        1 : "VIS",
        2 : "MOT",
        3 : "PCIN",
        4 : "MF",
        5 : "ACIN",
        6 : "ORF",
        7 : "STEMP",
        8 : "IF",
        9 : "ANTIN",
        10 : "ATEMPP",
        11 : "VLT",
        12 : "SP",
        13 : "ANG",
        14 : "ERC",
        15 : "BA35",
        16 : "CA1",
        17 : "SUB",
        18 : "PHC",
        19 : "AACIN"
    }
    
    # Read the original MRI
    api = Convert3D()
    api.execute(f'{param.fn_input_mri} -popas MRI {param.fn_input_seg} -popas SEG')

    # Start writing LaTeX file
    with open(param.fn_global_texfile(), 'wt') as latex:

        # Write the preample
        latex.write(textwrap.dedent(s_preamble))

        # Repeat for all slab images
        for i_slab in slab_indices:
            
            # Load the slab and reslice MRI to it
            api.execute(f'{param.fn_slab_with_dots_volume_image(i_slab)} -as slab '
                        f'-push MRI -reslice-identity -stretch 1% 99% 0 255 -clip 0 255 -as MRIslab '
                        f'-push SEG -reslice-identity -popas SEGslab')

            # Read the image into a NUMPY array
            slab = api.get_image('slab')
            slab_arr = sitk.GetArrayFromImage(slab)

            # Read the MRI slab
            mri_slab = api.get_image('MRIslab')
            mri_slab_arr = sitk.GetArrayFromImage(mri_slab)

            # Read the segmentation slab
            seg_slab = api.get_image('SEGslab')
            seg_slab_arr = sitk.GetArrayFromImage(seg_slab)

            # Get physical (RAS) coordinates of the voxels
            LPS_mat = np.array(slab.GetDirection()).reshape(3,3) @ np.diag(slab.GetSpacing())
            LPS_mat_inv = np.linalg.inv(LPS_mat)
            LPS_off = np.array(slab.GetOrigin())
            LPS_off_inv = - LPS_mat_inv @ LPS_off
            LPS_to_RAS_mat = np.diag([-1.,-1.,1.])
            idx_ijk = np.flip(np.vstack(np.where(np.isfinite(slab_arr))), axis=0)
            idx_ras = LPS_to_RAS_mat @ ((LPS_mat @ idx_ijk).T + LPS_off).T
            x_ras = idx_ras[0,:].reshape(slab_arr.shape)
            y_ras = idx_ras[1,:].reshape(slab_arr.shape)
            z_ras = idx_ras[2,:].reshape(slab_arr.shape)

            # Get the unique dots
            dot_vals = np.array([int(x) for x in np.unique(slab_arr[:]) if x not in [0, 255]])

            # Report
            print(f"Making figures for slab {i_slab}; Dots: {dot_vals}")

            # Get the voxel coordinates of dot centers
            dot_ctrs = np.zeros((dot_vals.size,3))
            for i,d in enumerate(dot_vals):
                dot_ctrs[i,:] = np.mean(idx_ras[:,slab_arr.flat==d], axis=1)
                
            # Get the RAS y coordinate of the most anterior dot
            y_anterior_most = slabs[i_slab]['y1'] - (param.dot_guide_depth if param.dot_guide_depth is not None else 0.2)
            if dot_ctrs.shape[0] > 0:
                y_anterior_most = np.max(dot_ctrs[:,1])
                
            # Map this coordinate to the slide index
            j_anterior_most = int(np.round((LPS_mat_inv @ LPS_to_RAS_mat @ np.array([0,y_anterior_most,0]) + LPS_off_inv)[1]))
            
            # Collapse the image along the y axis for 2D printing
            # slab_proj = np.mean(slab_arr > 0, axis=1)
            
            # Show slab at the level of the most anterior dot
            slab_proj = seg_slab_arr[:,j_anterior_most,:]
            
            # Get the MRI at the depth of the most anterior dot - this is where the cutting will
            # be taking place most likely

            # Get the first slice of the MRI slab
            mri_first_slab = mri_slab_arr[:,j_anterior_most,:]
            mri_first_slab[slab_proj == 0] = np.nan
            mri_first_slab = np.flip(mri_first_slab, 0)
            
            # mri_first_slab = np.rot90(mri_first_slab)

            # Get the physical coordinates for this projection
            x_proj,z_proj = x_ras[:,0,:], z_ras[:,0,:]

            # Get the anterior-posterior extents of the slab
            ext_anterior = np.amax(y_ras[:])
            ext_posterior = np.amin(y_ras[:])

            # Code for contour plots
            # Plot extents in mm
            figsz_mm = [80., 140.]
            x_ctr, z_ctr = (x_proj[0,0] + x_proj[-1,-1]) / 2, (z_proj[0,0] + z_proj[-1,-1]) / 2
            figext_mm = np.array([
                [ x_ctr - figsz_mm[0]*0.5, x_ctr + figsz_mm[0]*0.5 ],
                [ z_ctr - figsz_mm[1]*0.5, z_ctr + figsz_mm[1]*0.5 ] ])

            fn_out = ["",""]
            fig_width = [0,0]
            for flip in (0,1):
                fig = plt.figure(figsize=(figsz_mm[0]/(0.8*25.4),figsz_mm[1]/(0.9*25.4)))
                fig_width[flip] = fig.get_size_inches()[0]
                ax = fig.add_axes((0.15,0.075,0.8,0.9))
                plt.xlim(figext_mm[0,flip], figext_mm[0,1-flip])
                plt.ylim(figext_mm[1,0], figext_mm[1,1])

                plt.contour(x_proj, z_proj, slab_proj, (0.5,))
                # Show an overlay of the MRI to help visualize the sucli and gyri wrt dots
                plt.imshow(mri_first_slab, cmap='gray', alpha=0.65, extent=[x_proj.min(), x_proj.max(), z_proj.min(), z_proj.max()])
                #plt.scatter(dot_ctrs[:,0], dot_ctrs[:,2], c=dot_vals)
                #plt.gca().set_aspect('equal')

                mymarks = ['o','v','x','D','+','*']
                for i,d in enumerate(dot_vals):
                    dxy = dot_ctrs[i,[0,2]]
                    plt.scatter(dxy[0], dxy[1], color='r', marker=mymarks[i], s=40, label=f'Dot {int(d):d}', alpha=0.75, zorder=3)
                    plt.annotate("Dot " + str(d), dxy, dxy+10, weight='bold',
                                 path_effects=[pe.withStroke(linewidth=1, foreground="white")],
                                 arrowprops=dict(facecolor='black', arrowstyle="->", alpha=0.6))

                ax.xaxis.set_major_locator(matplotlib.ticker.MultipleLocator(10.))
                ax.xaxis.set_minor_locator(matplotlib.ticker.MultipleLocator(5.))
                ax.yaxis.set_major_locator(matplotlib.ticker.MultipleLocator(10.))
                ax.yaxis.set_minor_locator(matplotlib.ticker.MultipleLocator(5.))
                ax.grid(color="gray", which="major", linestyle=':', linewidth=0.5)
                ax.grid(color="gray", which="minor", linestyle=':', linewidth=0.25, alpha=0.5)

                if len(dot_vals) > 0:
                    ax.legend()

                # Output filename
                fn_out[flip] = param.fn_slab_tex_png(i_slab, flip)
                fig.savefig(fn_out[flip], dpi=300)
                fig.clear()
                plt.close(fig)

            # Generate code for the dots depth table
            s_table = ""
            if len(dot_vals) > 0:
                s_table += "\\begin{tabular}{|c|c|p{.8in}|p{.8in}|} \\hline\n"
                s_table += "    \\textbf{Dot} & \\textbf{Region} &\\textbf{Depth from Anterior} & \\textbf{Depth from Posterior} \\\\ \\hline\n"
                for i,d in enumerate(dot_vals):
                    s_table += "    Dot %d & %s & %6.2f mm & %6.2f mm \\\\ \\hline\n" % (
                        d, dot_names[d],
                        ext_anterior - dot_ctrs[i,1],
                        dot_ctrs[i,1] - ext_posterior)
                s_table += "    \\end{tabular}"
            s_table += "\\\\ \\vspace{5mm} \\small{MRI and contour shown {%5.2f}mm from anterior}" % (slabs[i_slab]['y1'] - y_anterior_most)

            latex.write(textwrap.dedent(s_slab % (param.id_string, i_slab,
                                                fig_width[0], os.path.basename(fn_out[0]),
                                                fig_width[1], os.path.basename(fn_out[1]),
                                                s_table)))

        latex.write(textwrap.dedent(s_closing))


class PaperTemplateLauncher:
    
    def __init__(self, parser):
        parser.add_argument("--work", "-w", required=True, help="Work directory for brainmold (will be created)")
        parser.set_defaults(func = lambda args : self.run(args))
        
    def run(self, args):
        make_paper_template(args.work)
    
